from aiohttp import ClientSession
from flespi_sdk.modules.item_with_metadata import ItemWithMetadata


class User(ItemWithMetadata):
    def __init__(
        self,
        realm_id: int,
        user_id: int,
        session: ClientSession,
        cid: int | None = None,
    ):
        """
        Initializes the RealmUsers class with a client instance.

        :param client: The client instance used to make API requests.
        """
        item_path = f"platform/realms/{realm_id}/users/{user_id}"
        super().__init__(
            parent_id=realm_id,
            id=user_id,
            item_path=item_path,
            session=session,
            cid=cid,
        )

    async def update_password(
        self,
        password: str,
    ) -> None:
        """
        Updates the password of the user.

        :param password: The new password for the user.
        """
        if len(password) < 16:
            raise ValueError("Password must be at least 16 characters long")

        async with self.session.put(
            f"/platform/realms/{self.parent_id}/users/{self.id}",
            json={"password": password},
            headers=self.get_headers(),
        ) as response:
            await self.get_result(response)



from aiohttp import ClientSession

from flespi_sdk.modules.realms.realm_home import RealmHome
from flespi_sdk.modules.realms.realm_token_params import RealmTokenParams
from flespi_sdk.modules.realms.users import Users
from flespi_sdk.modules.item_with_metadata import ItemWithMetadata


class Realm(ItemWithMetadata):
    """
    Represents a realm in the Flespi system.
    """

    def __init__(self, realm_id: int, session: ClientSession, cid: int | None = None):
        item_path = f"platform/realms/{realm_id}"
        super().__init__(
            parent_id=None, id=realm_id, item_path=item_path, session=session, cid=cid
        )
        self.users = Users(
            realm_id=realm_id,
            client_session=session,
            cid=cid,
        )

        self.home = RealmHome(realm_id=realm_id, session=session, cid=cid)
        self.token_params = RealmTokenParams(
            realm_id=realm_id, session=session, cid=cid
        )



from aiohttp import ClientSession

from flespi_sdk.modules.metadata import Metadata
from flespi_sdk.modules.mqtt import MQTT
from flespi_sdk.modules.realms import Realms
from flespi_sdk.modules.subaccounts import Subaccounts


class Account:
    def __init__(
        self, session: ClientSession | None = None, id: int | None = None
    ) -> None:
        self.id = id
        self.session = session or ClientSession("https://flespi.io")

        self._init_modules()

    async def stop(self) -> None:
        """
        Close the aiohttp session.
        """
        await self.session.close()

    async def set_token(self, token: str) -> None:
        await self._reset(token)

    async def realm_login(
        self,
        realm_public_id: str,
        realm_username: str,
        realm_password: str,
    ) -> None:
        """
        Login to a realm and set the Authorization header for subsequent requests.
        :param realm_public_id: Public ID of the realm.
        :param realm_username: Username for the realm.
        :param realm_password: Password for the realm.
        :raises Exception: If the login fails.
        """
        if not realm_public_id or not realm_username or not realm_password:
            raise ValueError("All parameters are required")

        async with self.session.post(
            f"/realm/{realm_public_id}/login",
            json={"username": realm_username, "password": realm_password},
        ) as response:
            if response.status != 200:
                raise Exception("Login failed")
            response_json = await response.json()

            token = response_json["result"][0]["token"]

            await self._reset(token)

    async def _reset(self, token: str | None = None) -> None:
        """
        Reset the account by resetting token and clearing the CID from all subsystems.
        """
        async with self.session.get(
            "/platform/customer", headers=dict(Authorization=f"FlespiToken {token}")
        ) as response:
            if response.status != 200:
                raise Exception("Failed to get account ID: " + str(response.status))
            response_json = await response.json()
            result = response_json["result"]
            id = result[0]["id"]
            if not id:
                raise Exception("Failed to get account ID: " + response_json)
            self.id = id
            self.session.headers["Authorization"] = f"FlespiToken {token}"
            self._init_modules()

    def _init_modules(self):
        self.metadata = Metadata("platform/customer", self.session, self.id)
        self.subaccounts = Subaccounts(self.session, self.id)
        self.realms = Realms(self.session, self.id)
        self.mqtt = MQTT(self.session, self.id)



# from aiohttp import ClientSession
# from flespi_sdk.modules.realms.users.user import User
# from flespi_sdk.modules.flespi_session import FlespiSession


# class ItemsContainer(FlespiSession):
#     """
#     The RealmUsers class provides methods to manage users in a specific realm.
#     It allows for the retrieval of user information and the management of user roles.
#     """

#     def __init__(
#         self,
#         items_path: str,
#         item_class: type,
#         client_session: ClientSession,
#         cid: int | None = None,
#     ):
#         """
#         Initializes the RealmUsers class with a client instance.

#         :param client: The client instance used to make API requests.
#         """
#         super().__init__(client_session, cid)
#         self.items_path = items_path
#         self.itemClass = item_class

#     async def list(self, selector: str = "all"):
#         """
#         List all users in the realm.
#         :param selector: The selector to filter users.
#         :return: A list of User objects.
#         """
#         async with self.session.get(
#             f"{self.items_path}/{selector}",
#             headers=self.get_headers(),
#         ) as response:
#             result = await self.get_result(response)
#             return [
#                 self.itemClass(self.realm_id, user["id"], self.session, self.cid)
#                 for user in result
#             ]



import aiohttp

from flespi_sdk.modules.realms.realm import Realm
from flespi_sdk.modules.flespi_session import FlespiSession


class Realms(FlespiSession):
    def __init__(self, session: aiohttp.ClientSession, cid: int | None = None):
        super().__init__(session, cid)

    async def get(self, id: int):
        async with self.session.get(
            f"/platform/realms/{id}", headers=self.get_headers()
        ) as response:
            result = await self.get_result(response)
            realm = result[0]
            return Realm(
                realm_id=realm["id"],
                session=self.session,
                cid=self.cid,
            )

    async def list(self, selector: str = "all"):
        async with self.session.get(
            f"/platform/realms/{selector}",
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            return [
                Realm(realm_id=realm["id"], session=self.session, cid=self.cid)
                for realm in result
            ]



from aiohttp import ClientSession
from flespi_sdk.modules.realms.users.user import User
from flespi_sdk.modules.flespi_session import FlespiSession


class Users(FlespiSession):
    """
    The RealmUsers class provides methods to manage users in a specific realm.
    It allows for the retrieval of user information and the management of user roles.
    """

    def __init__(
        self,
        realm_id: int,
        client_session: ClientSession,
        cid: int | None = None,
    ):
        """
        Initializes the RealmUsers class with a client instance.

        :param client: The client instance used to make API requests.
        """
        super().__init__(client_session, cid)
        self.realm_id = realm_id

    async def get(self, user_id: int) -> User:
        """
        Get a user by ID.
        :param user_id: The ID of the user to retrieve.
        :return: A User object.
        """
        async with self.session.get(
            f"/platform/realms/{self.realm_id}/users/{user_id}",
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            return User(self.realm_id, result[0]["id"], self.session, self.cid)

    async def list(self, selector: str = "all") -> list[User]:
        """
        List all users in the realm.
        :param selector: The selector to filter users.
        :return: A list of User objects.
        """
        async with self.session.get(
            f"/platform/realms/{self.realm_id}/users/{selector}",
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            return [
                User(self.realm_id, user["id"], self.session, self.cid)
                for user in result
            ]



import sys
import argparse
from flespi_sdk.account import Account


async def get_account(prog: str = sys.argv[0]) -> Account:
    parser = argparse.ArgumentParser(
        prog=prog,
        description="CLI for Flespi SDK",
        epilog=f"Examples:\n  {prog} <token>\n  {prog} <realm> <username> <password>",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument(
        "args",
        nargs="+",
        help=(
            "Arguments for login:\n"
            "  - Provide 1 argument: <token>\n"
            "  - Provide 3 arguments: <realm> <username> <password>"
        ),
    )

    args = parser.parse_args()

    if len(args.args) == 1:
        token = args.args[0]
        account = Account()
        try:
            await account.set_token(token)
        except Exception as e:
            await account.stop()
            raise e
        return account
    elif len(args.args) == 3:
        realm, username, password = args.args
        account = Account()
        try:
            await account.realm_login(realm, username, password)
        except Exception as e:
            await account.stop()
            raise e
        return account
    else:
        parser.error(
            "Invalid number of arguments. Provide 1 argument (token) or 3 arguments (realm, username, password)."
        )



from flespi_sdk.account import Account


def main() -> None:
    print("Hello from plespi!")


async def login_with_token(token: str) -> Account:
    """
    Login to flespi with token
    :param token: token
    :return: Account object
    """
    if not token:
        raise ValueError("Token is required")
    if not isinstance(token, str):
        raise ValueError("Token must be a string")

    account = Account()
    await account.set_token(token)
    return account


async def login_with_realm(
    realm_public_id: str, realm_username: str, realm_password: str
):
    """
    Login to flespi with realm
    :param realm_public_id: realm public id
    :param realm_username: realm username
    :param realm_password: realm password
    :return: Account object
    """
    if not realm_public_id:
        raise ValueError("Realm public id is required")
    if not isinstance(realm_public_id, str):
        raise ValueError("Realm public id must be a string")
    if not realm_username:
        raise ValueError("Realm username is required")
    if not isinstance(realm_username, str):
        raise ValueError("Realm username must be a string")
    if not realm_password:
        raise ValueError("Realm password is required")
    if not isinstance(realm_password, str):
        raise ValueError("Realm password must be a string")

    account = Account()
    await account.realm_login(realm_public_id, realm_username, realm_password)
    return account



from aiohttp import ClientSession
from flespi_sdk.modules.flespi_session import FlespiSession
from flespi_sdk.modules.metadata import Metadata


class ItemWithMetadata(FlespiSession):
    def __init__(
        self,
        parent_id: int | None,
        id: int,
        item_path: str,
        session: ClientSession,
        cid: int | None = None,
    ):
        super().__init__(session, cid)
        self.parent_id = parent_id
        self.id = id
        self.item_path = item_path
        self.metadata = Metadata(item_path, session, cid)

    async def get_name(self) -> str:
        """
        Get the name of the item.
        :return: The name of the item.
        """
        async with self.session.get(
            f"/{self.item_path}",
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            return result[0]["name"]



import aiohttp

from flespi_sdk.modules.flespi_session import FlespiSession


class Subaccounts(FlespiSession):
    def __init__(self, session: aiohttp.ClientSession, cid: int | None = None):
        super().__init__(session, cid)

    async def get(self, id: str):
        from flespi_sdk.account import Account

        async with self.session.get(
            f"/platform/subaccounts/{id},cid={self.cid}", headers=self.get_headers()
        ) as response:
            result = await self.get_result(response)
            subacc = result[0]
            return Account(
                self.session,
                id=subacc["id"],
            )

    async def list(self, selector: str = "all"):
        from flespi_sdk.account import Account

        async with self.session.get(
            f"/platform/subaccounts/{selector},cid={self.cid}",
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            return [Account(self.session, id=subacc["id"]) for subacc in result]



from aiohttp import ClientSession

from flespi_sdk.exception import FlespiException


class FlespiSession:
    def __init__(self, session: ClientSession, cid: int | None = None):
        self.cid = cid
        self.session = session

    def reset(self, cid: int):
        self.cid = cid

    def get_headers(self):
        if self.cid:
            return {"X-Flespi-CID": str(self.cid)}
        raise ValueError("CID is not set. Please set the CID before using this method.")

    async def get_result(self, response):
        if response.status != 200:
            raise FlespiException(
                status_code=response.status, errors=(await response.json())["errors"]
            )
        response_json = await response.json()
        result = response_json["result"]
        return result



from aiohttp import ClientSession
from flespi_sdk.modules.flespi_session import FlespiSession


class RealmHome(FlespiSession):
    def __init__(
        self,
        realm_id: int,
        session: ClientSession,
        cid: int | None = None,
    ):
        super().__init__(session, cid)
        self.realm_id = realm_id

    async def get(self):
        params = {"fields": "home"}
        async with self.session.get(
            f"/platform/realms/{self.realm_id}",
            params=params,
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            return result[0]["home"]

    async def set(self, data: dict):
        response = await self.session.put(
            f"/platform/realms/{self.realm_id}",
            json={"home": data},
            headers=self.get_headers(),
        )
        return await self.get_result(response)

    async def set_current_subaccount(self):
        data = {"type": 0}
        await self.set(data)

    async def set_existing_subaccount(self, subaccount_id: int):
        data = {"type": 1, "subaccount_id": subaccount_id}
        await self.set(data)

    async def set_create_new_subaccount(
        self, subaccount_id: int | None = None, limit_id: int | None = None
    ):
        data = {"type": 2}
        if subaccount_id:
            data["subaccount_id"] = subaccount_id
        if limit_id:
            data["limit_id"] = limit_id

        await self.set(data)



from aiohttp import ClientSession

from flespi_sdk.modules.flespi_session import FlespiSession


class RealmTokenParams(FlespiSession):
    def __init__(self, realm_id: int, session: ClientSession, cid: int | None = None):
        super().__init__(session, cid)
        self.realm_id = realm_id

    async def get(self):
        params = {"fields": "token_params"}
        async with self.session.get(
            f"platform/realms/{self.realm_id}",
            params=params,
            headers=self.get_headers(),
        ) as response:
            return (await self.get_result(response))[0]["token_params"]

    async def set(self, data: dict):
        response = await self.session.put(
            f"platform/realms/{self.realm_id}",
            json={"token_params": data},
            headers=self.get_headers(),
        )
        return await self.get_result(response)

    async def set_ttl(self, seconds: int):
        token_params = await self.get()
        if token_params["ttl"] == seconds:
            return
        token_params["ttl"] = seconds
        await self.set(token_params)

    async def set_ip_addresses(self, ip_addresses: list[str]):
        token_params = await self.get()
        ip_addresses_csv = ",".join(ip_addresses)
        if "ips" in token_params and token_params["ips"] == ip_addresses_csv:
            return
        token_params["ips"] = ip_addresses_csv
        await self.set(token_params)

    async def allow_all_origins(self):
        token_params = await self.get()
        if "origins" in token_params and token_params["origins"] == "*":
            return
        token_params["origins"] = "*"
        await self.set(token_params)

    async def set_origins(self, origins: list[str], allow_flespi_origins: bool = False):
        token_params = await self.get()
        token_params["origins"] = origins.copy()
        if allow_flespi_origins:
            token_params["origins"].append({"preset": "flespi"})
        await self.set(token_params)

    async def set_standard_access(self):
        token_params = await self.get()
        if token_params["access"]["type"] == 0:
            return
        token_params["access"] = {"type": 0}
        await self.set(token_params)

    async def set_master_access(self):
        token_params = await self.get()
        if token_params["access"]["type"] == 1:
            return
        token_params["access"] = {"type": 1}
        await self.set(token_params)

    async def set_acl_access(
        self, acl: list[dict] = [], templating: bool | None = None
    ):
        token_params = await self.get()
        access = {
            "type": 2,
            "acl": acl,
        }
        if templating is not None:
            access["templating"] = templating
        token_params["access"] = access
        await self.set(token_params)



import urllib.parse

import aiohttp
from flespi_sdk.modules.flespi_session import FlespiSession


class MQTT(FlespiSession):
    def __init__(self, session: aiohttp.ClientSession, cid: int | None = None):
        super().__init__(session, cid)

    async def list(self, topic: str):
        """
        Get messages from the specified MQTT topic.
        :param topic: The MQTT topic to get messages from.
        :return: List of messages from the specified topic.
        """
        params = {"fields": "cid,topic,payload,user_properties"}
        async with self.session.get(
            f"/mqtt/messages/{urllib.parse.quote_plus(topic)}",
            params=params,
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            direct_messages = [msg for msg in result if msg["cid"] == self.cid]
            for msg in direct_messages:
                del msg["cid"]
            return direct_messages

    async def get(self, topic: str):
        """
        Get a specific message from the specified MQTT topic.
        :param topic: The MQTT topic to get the message from.
        :return: The message from the specified topic.
        """
        msgs = await self.list(topic)
        if len(msgs) > 1:
            raise ValueError(
                f"Multiple messages found for topic '{topic}'. Use list() to get all messages."
            )
        elif len(msgs) == 0:
            raise ValueError(f"No messages found for topic '{topic}'.")
        else:
            return msgs[0]

    async def publish(
        self, topic: str, payload: str | None = None, retained: bool = False
    ):
        """
         Publish a message to the specified MQTT topic.
        :param topic: The MQTT topic to publish the message to.
        :param payload: The message payload.
        :param retained: Whether the message should be retained.
        """
        message = {
            "topic": topic,
            "retained": retained,
            "payload": payload,
        }
        async with self.session.post(
            "/mqtt/messages",
            json=message,
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            return result

    async def delete(self, topic: str):
        """
        Delete a message from the specified MQTT topic.
        :param topic: The MQTT topic to delete the message from.
        """
        async with self.session.delete(
            f"/mqtt/messages/{urllib.parse.quote_plus(topic)}",
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            return result



from dataclasses import dataclass


@dataclass
class FlespiError:
    code: int
    reason: str
    id: int | None = None
    selector: str | None = None


class FlespiException(Exception):
    def __init__(self, status_code: int, errors: list[dict]):
        self.status_code = status_code
        self.errors = [FlespiError(**error) for error in errors]



import aiohttp

from flespi_sdk.modules.flespi_session import FlespiSession


class Metadata(FlespiSession):
    def __init__(
        self, item_path, session: aiohttp.ClientSession, cid: int | None = None
    ):
        super().__init__(session, cid)
        self.item_path = item_path

    async def get(self) -> dict:
        """
        Get metadata for the current account.
        :return: Metadata as a dictionary.
        """
        params = {"fields": "metadata"}
        async with self.session.get(
            self.item_path, params=params, headers=self.get_headers()
        ) as response:
            result = await self.get_result(response)
            return result[0]["metadata"]

    async def set(self, metadata: dict) -> None:
        """ "
        "Set metadata for the current account.
        :param metadata: Metadata as a dictionary.
        """
        async with self.session.put(
            self.item_path,
            json={"metadata": metadata},
            headers=self.get_headers(),
        ) as response:
            await self.get_result(response)

    async def get_value(self, key_path: str):
        """
        Get a specific value from the metadata.
        :param key_path: The key path to the value in the metadata.
        :return: The value from the metadata.
        """
        metadata = await self.get()
        if not metadata:
            return None
        keys = key_path.split(".")
        value = metadata
        for key in keys:
            if key in value:
                value = value[key]
            else:
                return None
        return value

    async def set_value(self, key_path: str, value) -> None:
        """
        Set a specific value in the metadata.
        :param key_path: The key path to the value in the metadata.
        :param value: The value to set.
        """
        metadata = await self.get() or {}
        keys = key_path.split(".")
        metadata_level = metadata
        for key in keys[:-1]:
            if key not in metadata_level:
                metadata_level[key] = {}
            metadata_level = metadata_level[key]
        metadata_level[keys[-1]] = value
        await self.set(metadata)

    async def delete_value(self, key_path: str) -> None:
        """
        Delete a specific key from the metadata.
        :param key_path: The key path to the value in the metadata.
        """
        metadata = await self.get()
        if not metadata:
            return
        keys = key_path.split(".")
        metadata_level = metadata
        for key in keys[:-1]:
            if key in metadata_level:
                metadata_level = metadata_level[key]
            else:
                return None
        del metadata_level[keys[-1]]
        await self.set(metadata)
